@@ -176,11 +176,7 @@ void Fill3DHoneycomb::_fill_surface_single(
             }
         }
         Polylines chained = PolylineCollection::chained_path_from(
-#if SLIC3R_CPPVER >= 11
             std::move(polylines), 
-#else
-            polylines,
-#endif
             PolylineCollection::leftmost_point(polylines), false); // reverse allowed
         bool first = true;
         for (Polylines::iterator it_polyline = chained.begin(); it_polyline != chained.end(); ++ it_polyline) {
@@ -199,12 +195,7 @@ void Fill3DHoneycomb::_fill_surface_single(
                 }
             }
             // The lines cannot be connected.
-#if SLIC3R_CPPVER >= 11
-            polylines_out.push_back(std::move(*it_polyline));
-#else
-            polylines_out.push_back(Polyline());
-            std::swap(polylines_out.back(), *it_polyline);
-#endif
+            polylines_out.emplace_back(std::move(*it_polyline));
             first = false;
         }
     }
