@@ -51,6 +51,8 @@ public:
     std::string wkt() const;
     Points concave_points(double angle = PI) const;
     Points convex_points(double angle = PI) const;
+    // Projection of a point onto the polygon.
+    Point point_projection(const Point &point) const;
 };
 
 extern BoundingBox get_extents(const Polygon &poly);
@@ -89,8 +91,10 @@ inline void        polygons_append(Polygons &dst, Polygons &&src)
 
 inline void polygons_rotate(Polygons &polys, double angle)
 {
-    for (Polygons::iterator p = polys.begin(); p != polys.end(); ++p)
-        p->rotate(angle);
+    const double cos_angle = cos(angle);
+    const double sin_angle = sin(angle);
+    for (Polygon &p : polys)
+        p.rotate(cos_angle, sin_angle);
 }
 
 inline Points to_points(const Polygon &poly)
@@ -150,7 +154,6 @@ inline Polylines to_polylines(const Polygons &polys)
     return polylines;
 }
 
-#if SLIC3R_CPPVER >= 11
 inline Polylines to_polylines(Polygons &&polys)
 {
     Polylines polylines;
@@ -164,7 +167,6 @@ inline Polylines to_polylines(Polygons &&polys)
     assert(idx == polylines.size());
     return polylines;
 }
-#endif
 
 } // Slic3r
 
